function C = strassen_3628(A, B, nmin)
%strassen_3628  Strassen's fast matrix multiplication algorithm.
%               C = strassen_3628(A, B, NMIN), where A and B are matrices of dimension
%               a power of 2, computes the product C = A*B.
%               Strassen's algorithm is used recursively until dimension <= NMIN
%               is reached, at which point standard multiplication is used.
%               The default is NMIN = 8 (which minimizes the total number of
%               operations).

%               Reference:
%               V. Strassen, Gaussian elimination is not optimal,
%               Numer. Math., 13 (1969), pp. 354-356.

if nargin < 3, nmin = 8; end

n = length(A);
if n ~= 2^( log2(n) )
   error('The matrix dimension must be a power of 2.')
end

if n <= nmin
   C = A*B;
else
   m = n/2; i = 1:m; j = m+1:n;
   P1 = strassen_3628( A(i,i)+A(j,j), B(i,i)+B(j,j), nmin);
   P2 = strassen_3628( A(j,i)+A(j,j), B(i,i), nmin);
   P3 = strassen_3628( A(i,i), B(i,j)-B(j,j), nmin);
   P4 = strassen_3628( A(j,j), B(j,i)-B(i,i), nmin);
   P5 = strassen_3628( A(i,i)+A(i,j), B(j,j), nmin);
   P6 = strassen_3628( A(j,i)-A(i,i), B(i,i)+B(i,j), nmin);
   P7 = strassen_3628( A(i,j)-A(j,j), B(j,i)+B(j,j), nmin);
   C = [ P1+P4-P5+P7  P3+P5;  P2+P4  P1+P3-P2+P6 ];
end
